angular
    .module('mapa', [])
    .factory('mapa', function (validaciones,
                               $exceptionHandler,
                               $rootScope,
                               localStorageService) {


        var feature;





        function guardarUltimaPosicionCompleta(posicionCompleta) {
            // console.log("guardando ultima posicion completa:" + JSON.stringify(posicionCompleta));
            localStorageService.set("ultimaPosicionCompleta", posicionCompleta)
        }





        function obtenerUltimaPosicionCompleta() {
            return localStorageService.get("ultimaPosicionCompleta")
        }





        return {

            mapa: '',
            centrado: false, //guarda estado de centrado

            convertirArregloPosiciones: convertirArregloPosiciones,
            guardarCoordenadaActual: guardarCoordenadaActual,
            guardarUltimaPosicionCompleta: guardarUltimaPosicionCompleta,
            obtenerCoordenadaActual: obtenerCoordenadaActual,
            obtenerUltimaPosicionCompleta: obtenerUltimaPosicionCompleta,
            dibujarMultiLineas: dibujarMultiLineas,
            dibujarRuta: dibujarRuta,


            agregarFeature: function (source, feature) {
                if (!validaciones.validar_objeto_vacio(source).Estado || !validaciones.validar_objeto_vacio(feature).Estado) {
                    console.log("agregar feature undefined");
                    return;
                }
                source.addFeature(feature);

            },


            removerFeature: function (source, feature) {
                try {

                    if (!validaciones.validar_objeto_vacio(source).Estado || !validaciones.validar_objeto_vacio(feature).Estado) {
                        console.log("remover feature undefined");
                        return;
                    }

                    source.removeFeature(feature);
                } catch (e) {
                    console.log("ERROR: " + e);

                    //si entra dentro del catch lo cachea, la aplicacion no se rompe y lo delega al $exceptionHandler para que haga lo suyo
                    //el error lo tengo que pasar como parametro sino brinda poca informacion, si le paso otra cosa hace lo mismo

                    // $exceptionHandler(e);
                }

            },


            actualizarMapa: function (long, lat, icono) {
                this.grabarCentro(long, lat);
                this.centrarIcono(icono, long, lat);
            },


            crearSourceXYZ: function () {
                return new ol.source.XYZ({
                    projection: this.obtenerProyeccion(),
                    minZoom: 1,
                    maxZoom: 18,
                    tileUrlFunction: function (coordinate) {
                        // Resto una unidad al Z para ir a la carpeta nuestra. Si piden z1 , devolvemos z0
                        var z = coordinate[0] - 1;

                        // Calculo la cantidad total de tiles verticales y se lo sumo a las coordenadas que me envian
                        var y = Math.pow(2, z) + coordinate[2];
                        return 'http://d3qooq5n8z7muo.cloudfront.net/map/1.0.0/basic/z' + z + '/x' + coordinate[1] + '/y' + y + '.jpg';
                    }
                });

            },

            crearIconosDeMoviles: function (array, ruta, source) {

                console.log('creando moviles: ');
                for (var i = 0; i < array.length; i++) {
                    var movil = this.crearIcono(
                        array[i]['lon'],
                        array[i]['lat'],
                        ruta,
                        array[i]['id_movil'],
                        0.4
                    );

                    source.addFeature(movil);

                }
            },

            grabarIconoCentral: function (longitud, latitud, rutaIcono, source) {
                console.log('grabando icono central: ');

                if (rutaIcono != undefined) {
                    var icono = this.crearIcono(
                        longitud,
                        latitud,
                        rutaIcono,
                        'casa',
                        0.4
                    );

                    //agrega el icono central a la capa iconos
                    source.addFeature(icono);
                }
            },

            obtenerPosicion: function (respuestaPosicion) {
                navigator.geolocation.getCurrentPosition(onSuccess, onError, {
                    maximumAge: 9000,
                    enableHighAccuracy: true
                });





                // funcion invocada cuando se obtiene exitosamente la posicion
                function onSuccess(position) {

                    var posiciones = {

                        'latitud': position.coords.latitude,
                        'longitud': position.coords.longitude

                    };

                    var rta = {
                        Estado: true,
                        Respuesta: posiciones
                    };

                    respuestaPosicion(rta);

                }





                //funcion invocada cuando no se puede obtener la posicion
                function onError(error) {
                    console.log('error en la obtencion de la posicion: ' + error.code + '\n' +
                        'message: ' + error.message + '\n');
                }
            },


            obtenerCantidadPixelesConZoom: function (zoom) {
                if (!validaciones.validar_vacio_null_y_undefined(zoom).Estado) {
                    console.log("obtener cantidad pixeles con zoom undefined");
                    return;
                }
                return Math.pow(2, (zoom + 1)) * 256;
            },


            obtenerMapa: function () {
                return mapa;
            },

            crearMapa: function (vista, target, capas) {

                if (capas == undefined) {
                    capas = [];
                }

                if (target == undefined) {
                    target = 'map';
                }

                //console.log('target: ' + target);

                mapa = new ol.Map({

                    layers: capas,
                    target: target,
                    view: vista

                });

                return mapa

            },


            grabarTarget: function (elemento) {
                mapa.setTarget(elemento);
            },

            //crea solo el estilo
            crearEstiloDeIcono: function (ruta, escala, ubicacion) {
                if (escala == undefined) {
                    escala = 1;
                }

                if (ubicacion == undefined) {
                    ubicacion = [0.5, 0.5];
                }

                return new ol.style.Style({
                    image: new ol.style.Icon(({
                        src: ruta,
                        scale: escala,
                        anchor: ubicacion

                    }))
                });
            },

            //crea el punto, setea el estilo
            crearIcono: function (longitud, latitud, ruta, nombre, escala, ubicacion) {
                if (!validaciones.validar_vacio_null_y_undefined(longitud).Estado || !validaciones.validar_vacio_null_y_undefined(latitud).Estado ||
                    validaciones.es_cero(latitud).Estado ||
                    validaciones.es_cero(longitud).Estado) {
                    console.log("crear icono undefined");
                    return;
                }

                var feature = new ol.Feature({
                    geometry: this.crearPunto(longitud, latitud)

                });

                feature.setStyle(this.crearEstiloDeIcono(ruta, escala, ubicacion));

                if (validaciones.validar_vacio_null_y_undefined(nombre).Estado) {
                    feature.setId(nombre);
                }

                return feature;
            },

            crearLinea: function (lon1, lat1, lon2, lat2) {

                if (!validaciones.validar_vacio_null_y_undefined(lon1).Estado || !validaciones.validar_vacio_null_y_undefined(lat1).Estado || !validaciones.validar_vacio_null_y_undefined(lon2).Estado || !validaciones.validar_vacio_null_y_undefined(lat2).Estado ||
                    validaciones.es_cero(lat1).Estado ||
                    validaciones.es_cero(lon1).Estado ||
                    validaciones.es_cero(lat2).Estado ||
                    validaciones.es_cero(lon2).Estado) {
                    console.log("crear linea undefined");
                    return;
                }

                var array = [[lon1, lat1], [lon2, lat2]];
                //var array = [[-60.653226, -32.9333933], [-60.654097, -32.9334272]];

                var linea = new ol.Feature({
                    geometry: new ol.geom.LineString(array)
                });

                return linea;
            },

            obtenerLatLon: function () {
                var latitud = mapa.getView().getCenter()[1];
                var longitud = mapa.getView().getCenter()[0];
                console.log('lat: ' + latitud);
                console.log('lon: ' + longitud);

                return [latitud, longitud];
            },

            crearFeature: function () {
                feature = new ol.Feature({});
            },

            obtenerFeature: function () {
                return feature;
            },

            grabarGeometria: function (geometria) {
                feature.setGeometry(geometria);
            },

            crearPunto: function (lat1, lon1) {
                if (!validaciones.validar_vacio_null_y_undefined(lat1).Estado || !validaciones.validar_vacio_null_y_undefined(lon1).Estado ||
                    validaciones.es_cero(lat1).Estado ||
                    validaciones.es_cero(lon1).Estado) {
                    console.log("crear punto undefined");
                    return;
                }
                return new ol.geom.Point([lat1, lon1])
            },

            crearEstilo: function () {
                return new ol.style.Style({});
            },

            obtenerEstilo: function () {

            },

            grabarCentro: function (long, lat, mapa) {
                if (!validaciones.validar_vacio_null_y_undefined(long).Estado || !validaciones.validar_vacio_null_y_undefined(lat).Estado ||
                    validaciones.es_cero(lat).Estado ||
                    validaciones.es_cero(long).Estado) {
                    console.log("grabar icono central undefined");
                    return;
                }

                mapa.getView().setCenter([long, lat]);

            },

            obtenerCentro: function () {
                console.log('centrooooooooooooooooooooooooooooo: ' + mapa.getView().getCenter());
                return mapa.getView().getCenter();
            },

            grabarZoom: function (zoom, mapa) {

                // console.log("grabando zoom: ");
                mapa.getView().setZoom(zoom);
            },

            obtenerZoom: function (mapa) {
                if (!validaciones.validar_vacio_null_y_undefined(mapa).Estado) {
                    console.log("grabar zoom undefined");
                    return;
                }
                return mapa.getView().getZoom()
            },

            centrarIcono: function (icono, long, lat) {
                if (!validaciones.validar_vacio_null_y_undefined(long).Estado || !validaciones.validar_vacio_null_y_undefined(lat).Estado || !validaciones.validar_objeto_vacio(icono).Estado ||
                    validaciones.es_cero(lat).Estado ||
                    validaciones.es_cero(long).Estado) {
                    console.log("centrar icono undefined");
                    return;
                }

                icono.getGeometry().setCoordinates([long, lat]);

            },

            //limpia todos los sources
            limpiarMapa: function (sources) {


                if (sources.length != 0) {
                    for (var i = 0; i < sources.length; i++) {
                        if (sources[i] != undefined) {
                            sources[i].clear();
                        }
                    }
                }
            },

            //actualiza la linea
            actualizarLinea: function (linea, arreglo) {

                if (!validaciones.validar_objeto_vacio(linea).Estado || !validaciones.lista_no_vacio(arreglo).Estado) {
                    console.log("actualizar linea es undefined");
                    return;
                }

                linea.getGeometry().setCoordinates(arreglo);

            },

            //actualiza la posicion del movil
            actualizarMovil: function (longitud_movil, latitud_movil, id_movil, sourceIconos) {
                console.log('id de movil: ' + id_movil);

                //actualiza la posicion del movil
                var feature = sourceIconos.getFeatureById(id_movil);
                feature.getGeometry().setCoordinates([longitud_movil, latitud_movil]);

            },

            crearVista: function (centro, zoom, proyeccion) {

                if (proyeccion == undefined) {
                    proyeccion = this.obtenerProyeccion();
                }

                if (zoom == undefined) {
                    zoom = 17;
                }

                if (centro == undefined) {
                    centro = [-60.71836709976196, -32.93372869491577]
                }

                return new ol.View({
                    projection: proyeccion,
                    center: centro,
                    zoom: zoom
                })
            },

            crearCapaVector: function (source) {
                /** @namespace ol.layer.Vector */
                if (source == undefined) {
                    source = [];
                }

                var vector = new ol.layer.Vector({
                    source: source
                });

                return vector;

            },


            crearCapaTile: function (source) {
                /** @namespace ol.layer.Tile */

                if (source == undefined) {
                    source = [];
                }

                var tile = new ol.layer.Tile({
                    source: source

                });

                return tile;

            },

            crearSourceVector: function (features) {
                if (features == undefined) {
                    features = [];
                }

                return new ol.source.Vector({
                    features: features
                })
            },

            removerCapa: function (capa) {
                mapa.removeLayer(capa)
            },

            obtenerDistanciaEntre2Puntos: function (lat1, lat2, lon1, lon2) {
                if (!validaciones.validar_vacio_null_y_undefined(lat1).Estado || !validaciones.validar_vacio_null_y_undefined(lat2).Estado || !validaciones.validar_vacio_null_y_undefined(lon1).Estado || !validaciones.validar_vacio_null_y_undefined(lon2).Estado ||
                    validaciones.es_cero(lat1).Estado ||
                    validaciones.es_cero(lon1).Estado ||
                    validaciones.es_cero(lat2).Estado ||
                    validaciones.es_cero(lon2).Estado) {
                    console.log("obtener distancia entre 2 puntos undefined");
                    return;
                }
                return Math.sqrt(Math.pow(lat2 - lat1, 2) + Math.pow(lon2 - lon1, 2));

            },

            obtenerProyeccion: function () {
                var pro = new ol.proj.Projection({
                    code: 'EPSG:4326',
                    units: 'm',
                    extent: [-180.0000, -90.0000, 180.0000, 90.0000]
                });

                ol.proj.addProjection(pro);

                return pro;

            },

            grabarCapa: function (capa) {
                mapa.addLayer(capa);
            },

            //obtiene todos las coordenadas de los puntos de una capa
            obtenerTodosLasCoordenadasDeUnaCapa: function (capa) {
                if (!validaciones.validar_vacio_null_y_undefined(capa).Estado) {
                    console.log("obtener todas las coordenadas de una capa undefined: ");
                    return;
                }
                return capa.getSource().getExtent();
            },

            //muestra todos los puntos que le pasen como argumento y ajusta el mapa
            muestraTodaLaCapaEnElMapa: function (coordenadas) {
                if (!validaciones.validar_vacio_null_y_undefined(coordenadas).Estado) {
                    console.log("muestra toda la capa en el mapa undefined: ");
                    return;
                }
                //setea centro entre los iconos, y tambien el zoom
                mapa.getView().fit(coordenadas, mapa.getSize());
            },

            //muestra todos los puntos en el mapa con el zoom justo para que entren todos
            ZoomAbarcaTodosLosPuntos: function (capa) {
                var coordenadas = this.obtenerTodosLasCoordenadasDeUnaCapa(capa);
                this.muestraTodaLaCapaEnElMapa(coordenadas);

            },

            obtenerAnchoPantalla: function (mapa) {

                return mapa.getSize()[0];
            },

            obtenerAltoPantalla: function (mapa) {
                return mapa.getSize()[1];
            },

            refrescarMapa: function () {
                if (mapa) {
                    mapa.updateSize();
                }
            },

            centrarEnUnPunto: function (lat1, latlejos, lon1, lonlejos, mapa) {

                if (!validaciones.validar_vacio_null_y_undefined(lat1).Estado || !validaciones.validar_vacio_null_y_undefined(latlejos).Estado || !validaciones.validar_vacio_null_y_undefined(lon1).Estado || !validaciones.validar_vacio_null_y_undefined(lonlejos).Estado ||
                    validaciones.es_cero(lat1).Estado ||
                    validaciones.es_cero(latlejos).Estado ||
                    validaciones.es_cero(lon1).Estado ||
                    validaciones.es_cero(lonlejos).Estado) {
                    console.log("zoom automatico undefined: ");
                    return;
                }

                var anchoPantalla = this.obtenerAnchoPantalla(mapa);
                var altoPantalla = this.obtenerAltoPantalla(mapa);
                var zoom = 17;
                var pixelesX = this.obtenerCantidadPixelesConZoom(zoom);
                var distancia = this.obtenerDistanciaEntre2Puntos(lat1, latlejos, lon1, lonlejos) * pixelesX / 360;

                if (altoPantalla < anchoPantalla) {
                    anchoPantalla = altoPantalla;
                }

                //console.log('pixeles: ' + pixelesX);
                //console.log('pixelesOcupadaps: ' + distancia);

                while (anchoPantalla < distancia) {
                    if (zoom == 0) {
                        break;
                    }
                    zoom--;
                    distancia = this.obtenerDistanciaEntre2Puntos(lat1, latlejos, lon1, lonlejos) * this.obtenerCantidadPixelesConZoom(zoom) / 360;
                    //console.log('zoomactual es:' + zoom);
                    //console.log('pixelsOcupados desde el centro a un punto:' + distancia);
                }

                this.grabarCentro(lat1, lon1, mapa);
                this.grabarZoom(zoom, mapa);


            },


            distanciaEntre2PuntosEnMetros: function (lat1, lon1, lat2, lon2) {
                //es la version mas eficiente que encontré en internet

                var deg2rad = 0.017453292519943295; // === Math.PI / 180
                var cos = Math.cos;
                lat1 *= deg2rad;
                lon1 *= deg2rad;
                lat2 *= deg2rad;
                lon2 *= deg2rad;
                var diam = 12742; // Diameter of the earth in km (2 * 6371)
                var dLat = lat2 - lat1;
                var dLon = lon2 - lon1;
                var a = (
                    (1 - cos(dLat)) +
                    (1 - cos(dLon)) * cos(lat1) * cos(lat2)
                ) / 2;

                return diam * Math.asin(Math.sqrt(a)) * 1000;
            }
        };





        function dibujarMultiLineas(source, posiciones) {


            console.error("entra dibujar lineas,posicines:" + posiciones);
            for (var i = 0, longitud = posiciones.length; i < longitud; i++) {
                var polyline = new ol.geom.LineString(posiciones[i]);

                var feature = new ol.Feature(polyline);

                source.addFeature(feature);
            }


        }





        function convertirArregloPosiciones(arregloPosiciones) {
            // quizas puede ser parte de la misma funcion que dibuja el recorrido, pero asi queda bien separado

            // var arreglos = [['0a', '0b'], ['1a', '1b'], ['2a', '2b'], ['3a', '3b']];

            // salida = [
            //     [['0a', '0b'], ['1a', '1b']],
            //     [['1a', '1b'], ['2a', '2b']],
            //     [['2a', '2b'], ['3a', '3b']]
            // ];

            var ret = [];

            for (var i = 0, longitud = arregloPosiciones.length; i < longitud; i++) {

                ret.push([arregloPosiciones[i], arregloPosiciones[i + 1]]);

            }

            ret.pop(); //porque el ultimo elemento se repite
            return ret

        }





        //el server lo mada con informacion irrelevante. asi que lo limpia
        function limpiarDatosRecibidos(c) {
            var ar = [];

            for (var i = 0, longitud = c.length; i < longitud; i++) {
                ar.push([c[i][0], c[i][1]])
            }

            return ar;
        }





        function dibujarRuta(posiciones) {

            console.log("entra dibujar ruta");

            var esteServicio = this;
            var source = new ol.source.Vector();
            var vector = esteServicio.crearCapaVector(source);

            var ar = limpiarDatosRecibidos(posiciones);
            var retorno = convertirArregloPosiciones(ar);

            console.log("retorno es:" + retorno);

                vector.setStyle($rootScope.estiloVerde);


            esteServicio.grabarCapa(vector);

            dibujarMultiLineas(source, retorno);

        }





        function guardarCoordenadaActual(coordenada) {
            if (coordenada[0] != null && coordenada[1] != null) {
                localStorageService.set("latitud", coordenada[0]);
                localStorageService.set("longitud", coordenada[1]);
            }
        }





        function obtenerCoordenadaActual() {
            return [localStorageService.get("latitud"), localStorageService.get("longitud")]
        }
    });


